unit QueueDlg;

interface

uses
  Winapi.Windows,
  Winapi.Messages,
  System.SysUtils,
  System.Variants,
  System.Classes,
  Vcl.Graphics,
  Vcl.Controls,
  Vcl.Forms,
  Vcl.Dialogs,

  VaxTeleServerSIP, Vcl.ExtCtrls, Vcl.StdCtrls;

type
  TQueueForm = class(TForm)
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    comboQueueName: TComboBox;
    Label2: TLabel;
    ComboQueueProcessMood: TComboBox;
    GroupBox3: TGroupBox;
    RadiobtnFree: TRadioButton;
    RadiobtnAny: TRadioButton;
    Label3: TLabel;
    comboQueueAgents: TComboBox;
    ListQueueLogin: TListBox;
    btnAddAgent: TButton;
    btnRemoveAgent: TButton;
    btnClearAgent: TButton;
    btnAddQueue: TButton;
    btnRemoveQueue: TButton;
    btnClearDlg: TButton;

    procedure comboQueueNameChange(Sender: TObject);
    procedure btnAddQueueClick(Sender: TObject);
    procedure btnRemoveQueueClick(Sender: TObject);
    procedure btnRemoveAgentClick(Sender: TObject);
    procedure btnClearAgentClick(Sender: TObject);
    procedure btnAddAgentClick(Sender: TObject);

    private

    function AddQueueChecks(): boolean;
    function GetProcessMood(): Integer;
    function GetModeType() : Integer;

    procedure FillLogin();
    procedure FillQueue();

    procedure SetProcessMode();
    procedure ClearDlgContent();

    procedure UpdateDlgContent();

  public
    constructor Create(AOwner : TComponent; var pVaxTeleSIP : TVaxTeleServerSIP); reintroduce; overload;

  public
    procedure OnSetActive();

end;

var
  QueueForm: TQueueForm;
  m_pVaxTeleSIP : TVaxTeleServerSIP;

implementation

{$R *.dfm}

procedure TQueueForm.comboQueueNameChange(Sender: TObject);

var
  slistAgentName : TStringList;
  sAgentName : string;
  nModeId : Integer;
  nModeType : Integer;
  nIndex: Integer;

begin
        if(comboQueueName.Text = '') then
        begin
          Exit();
        end;

        slistAgentName := TStringList.Create();

       if(m_pVaxTeleSIP.GetQueueDetails(comboQueueName.Text, nModeId, nModeType, slistAgentName) = False) then
       begin
         Exit();
       end;

       ListQueueLogin.Clear();
       for sAgentName in slistAgentName do
        begin
         ListQueueLogin.Items.Add(sAgentName);
        end;

        ComboQueueProcessMood.ItemIndex := nModeId;

        RadiobtnFree.Checked := False;
        RadiobtnAny.Checked := False;

        if(nModeType = 1) then
          RadiobtnFree.Checked := True;
        if(nModeType = 2) then
           RadiobtnAny.Checked := True;
end;

constructor TQueueForm.create(AOwner : TComponent; var pVaxTeleSIP : TVaxTeleServerSIP);
begin
        inherited Create(AOwner);
        m_pVaxTeleSIP := pVaxTeleSIP;
    end;

procedure TQueueForm.OnSetActive();
begin
  UpdateDlgContent();
end;

function TQueueForm.AddQueueChecks() : boolean;
begin

  if(comboQueueName.Text  = '') then
  begin
    ShowMessage('Please enter Queue Name.');
    Exit(False);
  end;

  if(ComboQueueProcessMood.Text = '') then
  begin
    ShowMessage('Please select Process Mode');
    Exit(False);
  end;

 if((RadiobtnFree.Checked = False) and (RadiobtnAny.Checked = False))  then
  begin
    ShowMessage('Please select Process Mode Type Free/Any');
    Exit(False);
  end;

  Result := True;

end;

function TQueueForm.GetProcessMood(): Integer;
begin
   Result := ComboQueueProcessMood.ItemIndex;
end;

function TQueueForm.GetModeType() : Integer;
begin

  if(RadiobtnFree.Checked)  then
    begin
      Exit(1);
    end;

   Result := 2;
end;


procedure TQueueForm.FillLogin();
var
  objLoginList : TStringList;
  sAgentName : string;

begin

  objLoginList := TStringList.Create();
  m_pVaxTeleSIP.GetUserAll(objLoginList);

  comboQueueAgents.Clear();

  for sAgentName in objLoginList do
    begin
      comboQueueAgents.Items.Add(sAgentName);
    end;

    comboQueueAgents.ItemIndex := 0;
end;

procedure TQueueForm.FillQueue();
var
  objQueueList : TStringList;
  sQueueName : string;

begin

  if(comboQueueName.Items.Count > 0) Then
      comboQueueName.Clear;

  objQueueList := TStringList.Create();
  m_pVaxTeleSIP.GetQueueList(objQueueList);

  for sQueueName in objQueueList do
    begin
      comboQueueName.Items.Add(sQueueName);
    end;

    comboQueueName.ItemIndex := 0;
end;

procedure TQueueForm.SetProcessMode();
begin

   ComboQueueProcessMood.Clear();
   ComboQueueProcessMood.Items.Add('Hunt Random');
   ComboQueueProcessMood.Items.Add('Round Robin');
   ComboQueueProcessMood.Items.Add('Least Call Count');
   ComboQueueProcessMood.Items.Add('Least Talk Time');
   ComboQueueProcessMood.Items.Add('Longest Waiting');
   ComboQueueProcessMood.Items.Add('Prioritized Hunt');

   ComboQueueProcessMood.ItemIndex := 0;
end;

procedure TQueueForm.btnAddAgentClick(Sender: TObject);
var
sAgentName : string;
begin
      sAgentName := comboQueueAgents.Text;
      ListQueueLogin.Items.Add(sAgentName);
end;

procedure TQueueForm.btnAddQueueClick(Sender: TObject);
var
    sQueueAgent : string;
    nIndex : Integer;
begin

	if(AddQueueChecks() = False) Then
  begin
    Exit();
  end;


	m_pVaxTeleSIP.RemoveQueue(comboQueueName.Text);

   if(m_pVaxTeleSIP.AddQueue(comboQueueName.Text) <> True) then
   begin
     ShowMessage('Queue could not be added.');
     Exit();
   end;


	for sQueueAgent in ListQueueLogin.Items do
  begin
         m_pVaxTeleSIP.AddQueueAgent(comboQueueName.Text, sQueueAgent);
  end;

  comboQueueName.Items.Add(comboQueueName.Text);


  m_pVaxTeleSIP.SetQueueProcessMode(comboQueueName.Text, GetProcessMood(), GetModeType());

  ShowMessage('Queue created successfully"');


end;

procedure TQueueForm.btnClearAgentClick(Sender: TObject);
begin

  ListQueueLogin.Clear();

end;

procedure TQueueForm.btnRemoveAgentClick(Sender: TObject);
var
nindex: Integer;
sAgentName  : string;
begin

    for nindex:= ListQueueLogin.items.count -1 downto 0 do
      if ListQueueLogin.selected[nindex] then
        ListQueueLogin.items.delete(nindex);
end;

procedure TQueueForm.btnRemoveQueueClick(Sender: TObject);
begin

   m_pVaxTeleSIP.RemoveQueue(comboQueueName.Text);
   ClearDlgContent();

   comboQueueName.items.delete(comboQueueName.ItemIndex);

    ShowMessage('Queue Removed Successfully!');

end;

procedure TQueueForm.ClearDlgContent();
begin

   comboQueueName.Text := '';
   ComboQueueProcessMood.Text := '';

   ListQueueLogin.Clear;

end;

procedure TQueueForm.UpdateDlgContent();
begin

   FillQueue();
   FillLogin();
   SetProcessMode();

end;

end.
