#include "stdafx.h"
#include "ConfRoomList.h"
#include "VaxServerExCOM.h"

CConfRoomList::CConfRoomList(CVaxServerExCOM* pServerExCOM)
{
	m_pServerExCOM = pServerExCOM;

	m_lConfRoomByNameListId = pServerExCOM->CustomListOpen(256);
	m_lConfRoomByNoListId = pServerExCOM->CustomListOpen(256);
}

CConfRoomList::~CConfRoomList()
{
	DeleteRoomAll();

	m_pServerExCOM->CustomListClose(m_lConfRoomByNameListId);
	m_pServerExCOM->CustomListClose(m_lConfRoomByNoListId);
}

void CConfRoomList::DeleteRoomAll()
{
	m_pServerExCOM->CustomListTrvsStart(m_lConfRoomByNameListId);

	for(int nCount = 0; nCount < m_pServerExCOM->CustomListCount(m_lConfRoomByNameListId); nCount++)
	{
		CConfRoomDetails* pConfRoom = (CConfRoomDetails*) m_pServerExCOM->CustomListTrvsNext(m_lConfRoomByNameListId);
		if(pConfRoom == NULL) continue;

		delete pConfRoom;
	}

	m_pServerExCOM->CustomListReset(m_lConfRoomByNameListId);
	m_pServerExCOM->CustomListReset(m_lConfRoomByNoListId);
}

CString CConfRoomList::GenKey(LPCTSTR sValue)
{
	CString sKey = sValue;
	sKey.MakeLower();

	return sKey;
}

CConfRoomDetails* CConfRoomList::FindRoomByName(LPCTSTR sRoomName)
{
	CString sKey = GenKey(sRoomName);
	return (CConfRoomDetails*) m_pServerExCOM->CustomListFindData(m_lConfRoomByNameListId, sKey);
}

CConfRoomDetails* CConfRoomList::FindRoomByNo(LPCTSTR sRoomNo)
{
	CString sKey = GenKey(sRoomNo);
	return (CConfRoomDetails*) m_pServerExCOM->CustomListFindData(m_lConfRoomByNoListId, sKey);
}

BOOL CConfRoomList::AddRoom(LPCTSTR sRoomName, LPCTSTR sRoomNo)
{
	CConfRoomDetails* pConfRoom = FindRoomByName(sRoomName);
	if(pConfRoom) return FALSE;

	pConfRoom = FindRoomByNo(sRoomNo);
	if(pConfRoom) return FALSE;

	pConfRoom = new CConfRoomDetails();

	pConfRoom->m_sRoomName = sRoomName;
	pConfRoom->m_sRoomNo   = sRoomNo;
				
	CString sKey = GenKey(sRoomName);
	m_pServerExCOM->CustomListAddData(m_lConfRoomByNameListId, sKey, pConfRoom);
		
	sKey = GenKey(sRoomNo);
	m_pServerExCOM->CustomListAddData(m_lConfRoomByNoListId, sKey, pConfRoom);
	
	return TRUE;
}

void CConfRoomList::RemoveRoom(LPCTSTR sRoomName)
{
	CConfRoomDetails* pConfRoom = FindRoomByName(sRoomName);
	if(pConfRoom == NULL) return;

	CString sKey = GenKey(pConfRoom->m_sRoomNo);
	m_pServerExCOM->CustomListRemoveData(m_lConfRoomByNoListId, sKey);

	sKey = GenKey(sRoomName);	
	m_pServerExCOM->CustomListRemoveData(m_lConfRoomByNameListId, sKey);

	delete pConfRoom;
}

CString CConfRoomList::GetRoomName(LPCTSTR sRoomNo)
{
	CConfRoomDetails* pConfRoom = FindRoomByNo(sRoomNo);
	if(pConfRoom == NULL) return "";

	return pConfRoom->m_sRoomName;
}

BOOL CConfRoomList::GetRoomDetails(LPCTSTR sRoomName, CString& sRoomNo)
{
	CConfRoomDetails* pConfRoom = FindRoomByName(sRoomName);
	if(pConfRoom == NULL) return FALSE;

	sRoomNo = pConfRoom->m_sRoomNo;
	return TRUE;
}

BOOL CConfRoomList::IsRoomNoExists(LPCTSTR sRoomNo)
{
	CConfRoomDetails* pConfRoom = FindRoomByNo(sRoomNo);
	if(pConfRoom == NULL) return FALSE;

	return TRUE;
}

BOOL CConfRoomList::GetRoomAll(CStringList* pRoomList)
{
	m_pServerExCOM->CustomListTrvsStart(m_lConfRoomByNameListId);

	for(int nCount = 0; nCount < m_pServerExCOM->CustomListCount(m_lConfRoomByNameListId); nCount++)
	{
		CConfRoomDetails* pConfRoom = (CConfRoomDetails*) m_pServerExCOM->CustomListTrvsNext(m_lConfRoomByNameListId);
		if(pConfRoom == NULL) continue;

		pRoomList->AddTail(pConfRoom->m_sRoomName);
	}

	return TRUE;
}
