#include "StdAfx.h"
#include "VaxServerExCOM.h"
#include "VaxCallSessionRoute.h"

//------------------------------------------------------------------------------------------------------------------
//-------------------------------			Class CDirectProxyRouteDetails
//------------------------------------------------------------------------------------------------------------------

CVaxCallSessionRoute::CVaxCallSessionRoute(CVaxServerExCOM* pServerExCOM)
{
	m_pServerExCOM = pServerExCOM;
	
	m_lUserRouteList  = pServerExCOM->CustomListOpen(256);
	m_lLineRouteList  = pServerExCOM->CustomListOpen(256);
	m_lProxyRouteList = pServerExCOM->CustomListOpen(256);
}

CVaxCallSessionRoute::~CVaxCallSessionRoute()
{
	m_pServerExCOM->CustomListClose(m_lUserRouteList);
	m_pServerExCOM->CustomListClose(m_lLineRouteList);
	m_pServerExCOM->CustomListClose(m_lProxyRouteList);
}

//------------------------------------------------------------------------------------------------------------------
//-------------------------------			Private Functions
//------------------------------------------------------------------------------------------------------------------

CString CVaxCallSessionRoute::GenKey(LPCTSTR sValue)
{
	CString sKey = sValue;
	sKey.MakeLower();
	return sKey;
}

CUserRouteDetails* CVaxCallSessionRoute::FindUserRoute(LPCTSTR sUserName)
{
	CString sKey = GenKey(sUserName);
	return (CUserRouteDetails*) m_pServerExCOM->CustomListFindData(m_lUserRouteList, sKey);
}

CLineRouteDetails* CVaxCallSessionRoute::FindLineRoute(LPCTSTR sLineName)
{
	CString sKey = GenKey(sLineName);
	return (CLineRouteDetails*) m_pServerExCOM->CustomListFindData(m_lLineRouteList, sKey);
}

CDirectProxyRouteDetails* CVaxCallSessionRoute::FindDirectProxyRoute(LPCTSTR sProxyName)
{
	CString sKey = GenKey(sProxyName);
	return (CDirectProxyRouteDetails*) m_pServerExCOM->CustomListFindData(m_lProxyRouteList, sKey);
}

void CVaxCallSessionRoute::DeleteAllUserRoutes()
{
	if(!m_pServerExCOM->CustomListTrvsStart(m_lUserRouteList))
		return;
	
	for(int nCount = 0; nCount < m_pServerExCOM->CustomListCount(m_lUserRouteList); nCount++)
	{
		CUserRouteDetails* pUserRouteDetails = (CUserRouteDetails*) m_pServerExCOM->CustomListTrvsNext(m_lUserRouteList);
		if(pUserRouteDetails == NULL) continue;

		delete pUserRouteDetails;
	}

	m_pServerExCOM->CustomListReset(m_lUserRouteList);
}

void CVaxCallSessionRoute::DeleteAllLineRoutes()
{
	if(!m_pServerExCOM->CustomListTrvsStart(m_lLineRouteList))
		return;
	
	while(TRUE)
	{
		CLineRouteDetails* pLineRouteDetails = (CLineRouteDetails*) m_pServerExCOM->CustomListTrvsNext(m_lLineRouteList);
		if(pLineRouteDetails == NULL) break;

		delete pLineRouteDetails;
	}

	m_pServerExCOM->CustomListReset(m_lLineRouteList);
}

void CVaxCallSessionRoute::DeleteAllProxyRoutes()
{
	if(!m_pServerExCOM->CustomListTrvsStart(m_lProxyRouteList))
		return;
	
	for(int nCount = 0; nCount < m_pServerExCOM->CustomListCount(m_lProxyRouteList); nCount++)
	{
		CDirectProxyRouteDetails* pDirectProxyRouteDetails = (CDirectProxyRouteDetails*) m_pServerExCOM->CustomListTrvsNext(m_lProxyRouteList);
		if(pDirectProxyRouteDetails == NULL) continue;

		delete pDirectProxyRouteDetails;
	}

	m_pServerExCOM->CustomListReset(m_lProxyRouteList);
}

BOOL CVaxCallSessionRoute::GetUserRouteDetails(LPCTSTR sUserName, CString& sUserPrefix, CString& sDialPrefix, int& nToPeerType, CString& sToPeerName)
{
	BOOL bResult = FALSE;
	
	CUserRouteDetails* pUserRouteDetails = FindUserRoute(sUserName);
	if(pUserRouteDetails == NULL) return FALSE;

	sUserPrefix = pUserRouteDetails->m_sUserPrefix;
	sDialPrefix = pUserRouteDetails->m_sDialPrefix;

	nToPeerType = pUserRouteDetails->m_nToPeerType;
	sToPeerName = pUserRouteDetails->m_sToPeerName;

	return TRUE;
}

BOOL CVaxCallSessionRoute::GetLineRouteDetails(LPCTSTR sLineName, int& nToPeerType, CString& sToPeerName, BOOL& bRejected, CString& sRoomName)
{
	CLineRouteDetails* pLineRouteDetails = FindLineRoute(sLineName);
	if(pLineRouteDetails == NULL) return FALSE;

	sRoomName      = pLineRouteDetails->m_sRoomName;
	sToPeerName    = pLineRouteDetails->m_sToPeerName;
	nToPeerType    = pLineRouteDetails->m_nToPeerType;
	bRejected      = pLineRouteDetails->m_bRejected;
	
	return TRUE;
}

BOOL CVaxCallSessionRoute::GetDirectProxyRouteDetails(LPCTSTR sProxyName, int& nToPeerType, CString& sToPeerName, BOOL& bRejected, CString& sRoomName)
{
	CDirectProxyRouteDetails* pDirectProxyRouteDetails = FindDirectProxyRoute(sProxyName);
	if(pDirectProxyRouteDetails == NULL) return FALSE;

	sRoomName      = pDirectProxyRouteDetails->m_sRoomName;
	sToPeerName    = pDirectProxyRouteDetails->m_sToPeerName;
	nToPeerType    = pDirectProxyRouteDetails->m_nToPeerType;
	bRejected      = pDirectProxyRouteDetails->m_bRejected;
		
	return TRUE;
}

//------------------------------------------------------------------------------------------------------------------
//-------------------------------			Events
//------------------------------------------------------------------------------------------------------------------

BOOL CVaxCallSessionRoute::OnCallFromUser(LPCTSTR sUserName, LPCTSTR sUserDialedNo, CString& sDialNo, int& nToPeerType, CString& sToPeerName)
{
	CString sUserPrefix, sDialPrefix;
	
	BOOL bResult = GetUserRouteDetails(sUserName, sUserPrefix, sDialPrefix, nToPeerType, sToPeerName);
	
	// USER-ROUTE not found. Make it U2U call.
	
	if(!bResult || nToPeerType == -1 || sToPeerName.IsEmpty())
	{
		sDialNo     = sUserDialedNo;
		nToPeerType = VAX_PEER_TYPE_USER;
		sToPeerName = sUserDialedNo;

		return TRUE;
	}

	int nUserPrefixLength = sUserPrefix.GetLength();
	
	// check if first 'N' charachters are equal to sUserPrefix. If YES; Modify it according to dialplan. Callee is on other server.
	
	CString sCallTo = sUserDialedNo;

	if( sCallTo.Left(nUserPrefixLength) == sUserPrefix )
	{
		sCallTo.Delete(0, nUserPrefixLength);
		sDialNo.Format("%s%s", sDialPrefix, sCallTo);
	}
	else
	{
		//Make it U2U call.

		sDialNo     = sUserDialedNo;
		nToPeerType = VAX_PEER_TYPE_USER;
		sToPeerName = sUserDialedNo;
	}

	return TRUE;
}

BOOL CVaxCallSessionRoute::OnCallFromLine(LPCTSTR sLineName, int& nToPeerType, CString& sToPeerName, BOOL& bRejected, CString& sRoomName)
{
	BOOL bResult = GetLineRouteDetails(sLineName, nToPeerType, sToPeerName, bRejected, sRoomName);
	if(!bResult) return FALSE;
	
	return TRUE;
}

BOOL CVaxCallSessionRoute::OnCallFromDirectProxySIP(LPCTSTR sProxyName, int& nToPeerType, CString& sToPeerName, BOOL& bRejected, CString& sRoomName)
{
	BOOL bResult = GetDirectProxyRouteDetails(sProxyName, nToPeerType, sToPeerName, bRejected, sRoomName);
	if(!bResult) return FALSE;

	return TRUE;
}

//------------------------------------------------------------------------------------------------------------------
//-------------------------------			Route (s)
//------------------------------------------------------------------------------------------------------------------

BOOL CVaxCallSessionRoute::AddUserRoute(LPCTSTR sUserName, LPCTSTR sUserPrefix, LPCTSTR sDialPrefix, int nToPeerType, LPCTSTR sToPeerName)
{
	CUserRouteDetails* pUserRouteDetails = FindUserRoute(sUserName);

	if(pUserRouteDetails == NULL)
	{
		pUserRouteDetails = new CUserRouteDetails();
		
		pUserRouteDetails->m_sUserPrefix = sUserPrefix;
		
		pUserRouteDetails->m_nToPeerType = nToPeerType;
		pUserRouteDetails->m_sToPeerName = sToPeerName;
				
		pUserRouteDetails->m_sDialPrefix = sDialPrefix;

		CString sKey = GenKey(sUserName);
		m_pServerExCOM->CustomListAddData(m_lUserRouteList, sKey, (HANDLE) pUserRouteDetails);

		return TRUE;
	}

	pUserRouteDetails->m_sUserPrefix = sUserPrefix;
	pUserRouteDetails->m_nToPeerType = nToPeerType;
	pUserRouteDetails->m_sToPeerName = sToPeerName;
	pUserRouteDetails->m_sDialPrefix = sDialPrefix;
		
	return TRUE;
}

void CVaxCallSessionRoute::RemoveUserRoute(LPCTSTR sUserName)
{
	BOOL bResult = FALSE;
	
	CUserRouteDetails* pUserRouteDetails = FindUserRoute(sUserName);
	if(pUserRouteDetails == NULL) return;

	CString sKey = GenKey(sUserName);
	m_pServerExCOM->CustomListRemoveData(m_lUserRouteList, sKey);
	
	delete pUserRouteDetails;
}

BOOL CVaxCallSessionRoute::AddLineRoute(LPCTSTR sLineName, int nToPeerType, LPCTSTR sToPeerName, BOOL bRejected, LPCTSTR sRoomName)
{
	CLineRouteDetails* pLineRouteDetails = FindLineRoute(sLineName);
	if(pLineRouteDetails == NULL)
	{
		pLineRouteDetails = new CLineRouteDetails();

		pLineRouteDetails->m_sLineName = sLineName;
		pLineRouteDetails->m_sRoomName = sRoomName;
		pLineRouteDetails->m_nToPeerType = nToPeerType;
		pLineRouteDetails->m_sToPeerName = sToPeerName;
		pLineRouteDetails->m_bRejected = bRejected;
		
		CString sKey = GenKey(sLineName);
		m_pServerExCOM->CustomListAddData(m_lLineRouteList, sKey, (HANDLE) pLineRouteDetails);

		return TRUE;
	}

	pLineRouteDetails->m_sLineName = sLineName;
	pLineRouteDetails->m_sRoomName = sRoomName;
	pLineRouteDetails->m_nToPeerType = nToPeerType;
	pLineRouteDetails->m_sToPeerName = sToPeerName;
	pLineRouteDetails->m_bRejected = bRejected;

	return TRUE;
}

void CVaxCallSessionRoute::RemoveLineRoute(LPCTSTR sLineName)
{
	CLineRouteDetails* pLineRouteDetails = FindLineRoute(sLineName);
	if(pLineRouteDetails == NULL) return;

	CString sKey = GenKey(sLineName);
	m_pServerExCOM->CustomListRemoveData(m_lLineRouteList, sKey);

	delete pLineRouteDetails;
}

BOOL CVaxCallSessionRoute::AddDirectProxyRoute(LPCTSTR sProxyName, int nToPeerType, LPCTSTR sToPeerName, BOOL bRejected, LPCTSTR sRoomName)
{
	CDirectProxyRouteDetails* pDirectProxyRouteDetails = FindDirectProxyRoute(sProxyName);
	if(pDirectProxyRouteDetails == NULL)
	{
		pDirectProxyRouteDetails = new CDirectProxyRouteDetails();

		pDirectProxyRouteDetails->m_sProxyName     = sProxyName;
		pDirectProxyRouteDetails->m_sRoomName      = sRoomName;
		pDirectProxyRouteDetails->m_sToPeerName    = sToPeerName;
		pDirectProxyRouteDetails->m_nToPeerType    = nToPeerType;
		pDirectProxyRouteDetails->m_bRejected      = bRejected;
		
		CString sKey = GenKey(sProxyName);
		m_pServerExCOM->CustomListAddData(m_lProxyRouteList, sKey, (HANDLE) pDirectProxyRouteDetails);

		return TRUE;
	}

	pDirectProxyRouteDetails->m_sProxyName     = sProxyName;
	pDirectProxyRouteDetails->m_sRoomName      = sRoomName;
	pDirectProxyRouteDetails->m_sToPeerName    = sToPeerName;
	pDirectProxyRouteDetails->m_nToPeerType    = nToPeerType;
	pDirectProxyRouteDetails->m_bRejected      = bRejected;

	return TRUE;
}

void CVaxCallSessionRoute::RemoveDirectProxyRoute(LPCTSTR sProxyName)
{
	CDirectProxyRouteDetails* pDirectProxyRouteDetails = FindDirectProxyRoute(sProxyName);
	if(pDirectProxyRouteDetails == NULL) return;

	CString sKey = GenKey(sProxyName);
	m_pServerExCOM->CustomListRemoveData(m_lProxyRouteList, sKey);

	delete pDirectProxyRouteDetails;
}
	
void CVaxCallSessionRoute::ResetOBJ()
{
	DeleteAllUserRoutes();
	DeleteAllLineRoutes();
	DeleteAllProxyRoutes();
}